#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/04/16
#Last Modify: 2018/04/16
# deeper learning insights machine learning page 183

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Load the data
df = pd.read_csv('/home/egg/projects/deeper_insights_machine_learning/wdbc.data',
header=None)
# Next, we assign the 30 features to a NumPy array X. Using LabelEncoder, we
# transform the class labels from their original string representation (M and
# B) into integers:
from sklearn.preprocessing import LabelEncoder
X = df.loc[:, 2:].values
y = df.loc[:, 1].values
le = LabelEncoder()
y = le.fit_transform(y)
print le.transform(['M', 'B'])

# Split
from sklearn.cross_validation import train_test_split
X_train, X_test, y_train, y_test = train_test_split(X, y, 
                                                    test_size=0.20,
                                                    random_state=1)
# Create the pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
from sklearn.linear_model import LogisticRegression
from sklearn.pipeline import Pipeline
pipe_lr = Pipeline([('scl', StandardScaler()),
                    ('clf', LogisticRegression(
                    penalty='l2',
                    random_state=0))])


# Learning Curve ##############################################################
from sklearn.learning_curve import learning_curve
# Via the train_sizes parameter in the learning_curve function, we can control
# the absolute or relative number of training samples that are used to generate
# the learning curves. Here, we set train_sizes=np.linspace(0.1, 1.0, 10) to
# use 10 evenly spaced relative intervals for the training set sizes. By
# default, the learning_curve function uses stratified k-fold cross-validation
# to calculate the cross-validation accuracy, and we set k = 10 via the cv
# parameter. 
train_sizes, train_scores, test_scores \
  = learning_curve(estimator=pipe_lr,
                   X=X_train,
                   y=y_train,
                   train_sizes=np.linspace(0.1, 1.0, 10),
                   cv=10, #k folds
                   n_jobs=1) #cpu cores
train_mean = np.mean(train_scores, axis=1)
train_std = np.std(train_scores, axis=1)
test_mean = np.mean(test_scores, axis=1)
test_std = np.std(test_scores, axis=1)
plt.plot(train_sizes,
         train_mean,
         color='blue',
         marker='o',
         markersize=5,
         label='training accuracy')
plt.fill_between(train_sizes,
                 train_mean + train_std,
                 train_mean - train_std,
                 alpha=0.15,
                 color='blue')
plt.plot(train_sizes,
         test_mean,
         color='green',
         linestyle='--',
         marker='s',
         markersize=5,
         label='validation accuracy')
plt.fill_between(train_sizes,
                 test_mean + test_std,
                 test_mean - test_std,
                 alpha=0.15,
                 color='green')
plt.grid()
plt.xlabel('Number of training samples')
plt.ylabel('Accuracy')
plt.legend(loc='lower right')
plt.ylim([0.8, 1.0])
plt.show()


# Validation Curve ############################################################
from sklearn.learning_curve import validation_curve
param_range = [0.001, 0.01, 0.1, 1.0, 10.0, 100.0]
# we specified the parameter that we wanted to evaluate. In this case, it is C,
# the inverse regularization parameter of the LogisticRegression classifier,
# which we wrote as 'clf__C' to access the LogisticRegression object inside the
# scikit-learn pipeline for a specified value range that we set via the
# param_range parameter
train_scores, test_scores = validation_curve(estimator=pipe_lr,
                                             X=X_train,
                                             y=y_train,
                                             param_name='clf__C',
                                             param_range=param_range,
                                             cv=10)
train_mean = np.mean(train_scores, axis=1)
train_std = np.std(train_scores, axis=1)
test_mean = np.mean(test_scores, axis=1)
test_std = np.std(test_scores, axis=1)
plt.plot(param_range, train_mean,
         color='blue',
         marker='o',
         markersize=5,
         label='training accuracy')
plt.fill_between(param_range,
                 train_mean + train_std,
                 train_mean - train_std,
                 alpha=0.15,
                 color='blue')
plt.plot(param_range, test_mean,
         color='green',
         linestyle='--',
         marker='s',
         markersize=5,
         label='validation accuracy')
plt.fill_between(param_range,
                 test_mean + test_std,
                 test_mean - test_std,
                 alpha=0.15,
                 color='green')
plt.grid()
plt.xscale('log')
plt.legend(loc='lower right')
plt.xlabel('Parameter C')
plt.ylabel('Accuracy')
plt.ylim([0.8, 1.0])
plt.show()
# Although the differences in the accuracy for varying values of C are subtle,
# we can see that the model slightly underfits the data when we increase the
# regularization strength (small values of C). However, for large values of C,
# it means lowering the strength of regularization, so the model tends to
# slightly overfit the data. In this case, the sweet spot appears to be around
# C=0.1.
